<?php
/**
 * Ipaddress.
 *
 * @package HostCMS
 * @version 7.x
 * @author Hostmake LLC
 * @copyright © 2005-2024 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
require_once('../../../../bootstrap.php');

Core_Auth::authorization($sModule = 'ipaddress');

// Код формы
$iAdmin_Form_Id = 385;
$sAdminFormAction = '/admin/ipaddress/visitor/filter/index.php';

$oAdmin_Form = Core_Entity::factory('Admin_Form', $iAdmin_Form_Id);

// Контроллер формы
$oAdmin_Form_Controller = Admin_Form_Controller::create($oAdmin_Form);
$oAdmin_Form_Controller
	->module(Core_Module_Abstract::factory($sModule))
	->setUp()
	->path($sAdminFormAction)
	->title(Core::_('Ipaddress_Filter.title'))
	->pageTitle(Core::_('Ipaddress_Filter.title'));

// Меню формы
$oAdmin_Form_Entity_Menus = Admin_Form_Entity::factory('Menus');

// Элементы меню
$oAdmin_Form_Entity_Menus->add(
	Admin_Form_Entity::factory('Menu')
		->name(Core::_('Admin_Form.add'))
		->icon('fa fa-plus')
		->href(
			$oAdmin_Form_Controller->getAdminActionLoadHref($oAdmin_Form_Controller->getPath(), 'edit', NULL, 0, 0)
		)
		->onclick(
			$oAdmin_Form_Controller->getAdminActionLoadAjax($oAdmin_Form_Controller->getPath(), 'edit', NULL, 0, 0)
		)
);

if (Core_Array::getPost('add_filter'))
{
	ob_start();

	Admin_Form_Entity::factory('Select')
		->name('type[]')
		->options(Ipaddress_Visitor_Filter_Controller::getTypes())
		->divAttr(array('class' => 'col-xs-12 col-sm-2 property-data'))
		->onchange('$.changeIpaddressFilterOption(this)')
		->controller($oAdmin_Form_Controller)
		->execute();

	Admin_Form_Entity::factory('Input')
		->name('get_name[]')
		->divAttr(array('class' => 'col-xs-12 col-sm-1 property-data ipaddress-filter-get-name hidden'))
		->placeholder(Core::_('Ipaddress_Filter.placeholder_name'))
		->controller($oAdmin_Form_Controller)
		->execute();

	Admin_Form_Entity::factory('Select')
		->name('condition[]')
		->options(Ipaddress_Visitor_Filter_Controller::getConditions())
		->divAttr(array('class' => 'col-xs-12 col-sm-2 col-lg-1 property-data ipaddress-filter-condition'))
		->controller($oAdmin_Form_Controller)
		->execute();

	Admin_Form_Entity::factory('Input')
		->name('value[]')
		->divAttr(array('class' => 'col-xs-12 col-sm-4 col-lg-3 property-data ipaddress-filter-value'))
		->placeholder(Core::_('Ipaddress_Filter.placeholder_value'))
		->controller($oAdmin_Form_Controller)
		->execute();

	Admin_Form_Entity::factory('Select')
		->name('case_sensitive[]')
		->options(array(
			Core::_('Ipaddress_Filter.case_unsensitive'),
			Core::_('Ipaddress_Filter.case_sensitive')
		))
		->divAttr(array('class' => 'col-xs-12 col-sm-2 col-lg-1 property-data ipaddress-filter-case-sensitive'))
		->value(1)
		->controller($oAdmin_Form_Controller)
		->execute();

	Admin_Form_Entity::factory('Div')
		->class('col-xs-12 col-lg-3 property-data filter-days-wrapper')
		->add(
			Admin_Form_Entity::factory('Code')->html('
				<input type="text" class="form-control" name="times[]" value="1"/><span> ' . Core::_('Ipaddress_Visitor_Filter.times') . ' </span><input type="text" class="form-control" name="hours[]" value="3"/><span> ' . Core::_('Ipaddress_Visitor_Filter.hours') . '</span>
			')
		)
		->controller($oAdmin_Form_Controller)
		->execute();

	$aJSON = array(
		'status' => 'success',
		'html' => ob_get_clean()
	);

	Core::showJson($aJSON);
}

// Добавляем все меню контроллеру
$oAdmin_Form_Controller->addEntity($oAdmin_Form_Entity_Menus);

// Элементы строки навигации
$oAdmin_Form_Entity_Breadcrumbs = Admin_Form_Entity::factory('Breadcrumbs');

// Элементы строки навигации
$oAdmin_Form_Entity_Breadcrumbs->add(
	Admin_Form_Entity::factory('Breadcrumb')
		->name(Core::_('Ipaddress.show_ip_title'))
		->href(
			$oAdmin_Form_Controller->getAdminLoadHref('/admin/ipaddress/index.php', NULL, NULL, '')
		)
		->onclick(
			$oAdmin_Form_Controller->getAdminLoadAjax('/admin/ipaddress/index.php', NULL, NULL, '')
	)
)->add(
	Admin_Form_Entity::factory('Breadcrumb')
		->name(Core::_('Ipaddress_Visitor.title'))
		->href(
			$oAdmin_Form_Controller->getAdminLoadHref('/admin/ipaddress/visitor/index.php', NULL, NULL, '')
		)
		->onclick(
			$oAdmin_Form_Controller->getAdminLoadAjax('/admin/ipaddress/visitor/index.php', NULL, NULL, '')
	)
)->add(
	Admin_Form_Entity::factory('Breadcrumb')
		->name(Core::_('Ipaddress_Filter.title'))
		->href(
			$oAdmin_Form_Controller->getAdminLoadHref($oAdmin_Form_Controller->getPath(), NULL, NULL, '')
		)
		->onclick(
			$oAdmin_Form_Controller->getAdminLoadAjax($oAdmin_Form_Controller->getPath(), NULL, NULL, '')
	)
);

$oAdmin_Form_Controller->addEntity($oAdmin_Form_Entity_Breadcrumbs);

// Действие "Применить"
$oAdminFormActionApply = Core_Entity::factory('Admin_Form', $iAdmin_Form_Id)
	->Admin_Form_Actions
	->getByName('apply');

if ($oAdminFormActionApply && $oAdmin_Form_Controller->getAction() == 'apply')
{
	$oControllerApply = Admin_Form_Action_Controller::factory(
		'Admin_Form_Action_Controller_Type_Apply', $oAdminFormActionApply
	);

	// Добавляем типовой контроллер редактирования контроллеру формы
	$oAdmin_Form_Controller->addAction($oControllerApply);
}

// Действие редактирования
$oAdmin_Form_Action = Core_Entity::factory('Admin_Form', $iAdmin_Form_Id)
	->Admin_Form_Actions
	->getByName('edit');

if ($oAdmin_Form_Action && $oAdmin_Form_Controller->getAction() == 'edit')
{
	$oIpaddress_Visitor_Filter_Controller_Edit = Admin_Form_Action_Controller::factory(
		'Ipaddress_Visitor_Filter_Controller_Edit', $oAdmin_Form_Action
	);

	// Хлебные крошки для контроллера редактирования
	$oIpaddress_Visitor_Filter_Controller_Edit
		->addEntity(
			$oAdmin_Form_Entity_Breadcrumbs
		);

	// Добавляем типовой контроллер редактирования контроллеру формы
	$oAdmin_Form_Controller->addAction($oIpaddress_Visitor_Filter_Controller_Edit);
}

// Действие "Копировать"
$oAdminFormActionCopy = Core_Entity::factory('Admin_Form', $iAdmin_Form_Id)
	->Admin_Form_Actions
	->getByName('copy');

if ($oAdminFormActionCopy && $oAdmin_Form_Controller->getAction() == 'copy')
{
	$oControllerCopy = Admin_Form_Action_Controller::factory(
		'Admin_Form_Action_Controller_Type_Copy', $oAdminFormActionCopy
	);

	// Добавляем типовой контроллер редактирования контроллеру формы
	$oAdmin_Form_Controller->addAction($oControllerCopy);
}

// Источник данных 0
$oAdmin_Form_Dataset = new Admin_Form_Dataset_Entity(
	Core_Entity::factory('Ipaddress_Visitor_Filter')
);

// Добавляем источник данных контроллеру формы
$oAdmin_Form_Controller->addDataset(
	$oAdmin_Form_Dataset
);

// Показ формы
$oAdmin_Form_Controller->execute();